' main SYN4IT (SYN4.00)
' performs waveform synthesis and inversion for SYN4

DEFINT I-N: OPTION BASE 1

rem $INCLUDE: 'S4COMMON'
rem $INCLUDE: 'S4FUNCTN'

if not isyn4 then call notalone

 dim z2(nparmax),diag(nparmax), sumwtph(7)
 dim atau2(ntaumax,nsormax), taudamp(ntaumax,nsormax)
 dim f(3),df(3,3),vslow(2),pdly0(2),pdly(3),sorparel(7,nsormax)
 dim tmp(2*nesmax)

'-- internal seismogram scaling factor UCOR so that velocities are in km/s,
'   densities in g/cm3, amplitudes will be in cm
 ucor=1.0e-13
 blank$=string$(80,32)
 npage=2

'--- iterate
call scclear ': call scprint (1,1,"") : INPUT "Number of iterations (1 to 8) "; nit
if input1(1,1,"Number of iterations (1 to 8)",nit$,kfalse,ktrue,ktrue,numlist$(0,8))=27 then 2525
  if nit$="0" then 2525 else nit=val(nit$)

redim DVAR(nit)
redim KPAR(7,nsormax), KTAU(ntaumax,nsormax), IEVFLAG(nsormax)

'--- find number of parameters NPAR and assign places in parameter list
'    KPAR(i,j) = which free parameter is SORPAR(i,j)
'    KTAU(i,j) = which free parameter is ATAU(i,j)
   npar=0
  for isor=1 to nsor
     nparsor=0
     for j=1 to 7
      if damp(j,isor)>0 then 
        npar=npar+1 : kpar(j,isor)=npar
        nparsor=nparsor+1
      end if
     next
    if stfdamp(isor)>0 then 
       for j=1 to ntau(isor)
         npar=npar+1
         ktau(j,isor)=npar
         nparsor=nparsor+1
       next
    end if

'-- flag the event if it has no parameters to be determined
   if nparsor=0 then ievflag(isor)=kfalse else ievflag(isor)=ktrue

  next

'--- check that sum weights > 0
if nit>1 then
  swt=0.0
  for ista=1 to nsta
    if iflag(ista) then swt=swt+wt(ista)
  next
    if swt=0 then call scprint (22,10,"*** Sum of weights is zero *** ") : goto 2520
end if

'-- one screen is used for each subevent, the top 21 lines of the screen 
'    is stored in character string LINER$
'--  lines 22-25 are common to all displays

'-- permanent
    call scprintc(24,1,"OPTIONS:")
    call scprompt((0),10,"Q-quit"):call scprompt((0),18,"#-display event #")
    call scprompt((0),37,"H-help")
    call scprint(22,1,"MESSAGE:")

'--clear all screens
    redim liner$(nsor+1)
    ncharm=2000

'--- write help screen to LINER$(1)
    iscreen=1
    liner$(iscreen)=string$(ncharm,32)
call scprint1(1,1,string$(80,"-"))
call scprint1(21,1,string$(80,"-"))
call scprint1(1,58," SYN4    HELP SCREEN ")
call scprint1(21,37," Enter Source number for ITERATION SCREEN ")

call scprint1(3,1,"The iteration screen displays information about the model and statistics for ")
call scprint1(4,1,"each iteration. ")

call scprint1(5,1,"It - iteration number")
call scprint1(6,1,"Err - standard error for the parameter above it for the most recent model")
call scprint1(7,1,"UnwtdVAR - the unweighted variance of the amplitude residuals")
call scprint1(8,1,"WtdVAR   - the weighted variance of the amplitude residuals")
call scprint1(9,1,"R/D%  - the ratio in percent of the residual variance over the data variance")

call scprint1(11,1,"If there are multiple sources (subevents) one iteration screen is kept for ")
call scprint1(12,1,"each. Press the subevent number (1-6) to see the screen for that event.")

call scprint1(14,1,"The MESSAGE line will describe conditions that cause the iterating to ")
call scprint1(15,1,"terminate. Also if the variance reduction goes below the minimum, iteration ")
call scprint1(16,1,"will cease.")


'-- put labels on all screens
 for isor= 1 to nsor
    iscreen=isor+1
    liner$(iscreen)=string$(ncharm,32)
    call scprint1 (1,1,"Source")
    call scprint1 (1,14,"iterations     parameters")
    call scprint1 (1,47,"==== SYN4   ITERATION SCREEN ==")
    call scvint (1,10,3,nit)
    call scvint (1,25,3,npar)
    call scprint1 (2,1,"It Strike   Dip  Rake Depth   dTo  Dist    Az  Moment  UnwtdVAR  WtdVAR   R/D%")
    call scprint1 (12, 1, "It   STF element amplitudes in percent of total")
  next
    if nit>1 then call scprint2(npage) ELSE call scprint(1,1,"Source"+str$(npage-1)+"   1 iteration")

'*************** ITERATION LOOP

for itno=1 to nit

'-- set timer
   time1=timer

'--- flag ITLAST for last iteration
  itlast = (itno=nit) or (ivarstop)
'--- flag MKSTAT to enable statistics
  mkstat= (nit>1) or (nit=1 and krmsc)

'-- for 1 iteration do not redimension arrays or check bounds
if mkstat then
  if not fninlim(npar,1,nparmax) then _
     call scprint (22,10,"*** Invalid number of parameters")  : goto 2520
  redim b(npar,npar), b2(npar,npar), z(npar)
end if

depth1=sorpar(4,1)
nrec=0: wres2=0: res2=0: sumwt=0: neq2=0: npph=0
sumdswt=0 : sumdos=0

erase datvar, resvar, sumwtph

'******************* STATION LOOP

   for ista=1 to nsta

  if (not iflag(ista)) then 2090  'skip station

'--- Set bounding indices:      
'      NPH= # of upgoing phases
'      IMNO = # pts in Inst*Q*Tau curve (SP), for this station.
'      NINV = # pts to use in inversion from seismogram, starting at IPF2(ista)
'      NES  = # pts total in seismograms
'      NESP = # pts of ES to add; =NINV (or =NES on last iteration)
'      NSYN = # pts of direct phase to add
'      
     nes=nes(ista)
     noninv=(ninv(ista)<=1)
     iph=iph(ista)
     kpwave=fnpwave(ista)
     xmag= refmag(ista)   ''phmag(kphtype(ista))
     if kpwave then nph=2 else nph=1
     inst=inst(ista)
     imno=inpts(ista)
     ninv=ninv(ista) : n1=ndat(ista)-ipf2(ista) : ninv=fnamin(ninv,n1)
     neq2=neq2+ninv
     nesp=nes                    'number of ES points to read in
     nsyn=fnamin(imno,nes)       'number of points in direct phase to add
     nphes=iphes(ista)
     isprec1 = index(ista)       'first record in SP array
     dt=dt(ista)
     nv=ninv+1
     nph1=nph+1
     wt=wt(ista)

'--- except for last iteration, only do NINV pts in synthetics
if (not itlast) then
  if (wt(ista)=0) or noninv or nodat(ista) then 2090  'skip this station
  nesp=fnamin(ninv+1,nes) : nsyn=fnamin(ninv+1,imno)  'cut to NINV if smaller
end if

  np=fnamax(1,npar) 
  redim d(nesp,np), es3(nesp,nph1), es2(nesp), r(nv)
  erase ss

'-- get the observed seismograms starting at point IPF2
   ipof=ipf2(ista)-1
   if not noninv and mkstat then 
     call s4synio(1,0,ista,ninv,ipof,odat(),dext(ista),datsq)
   end if
 
'-- calculate ray parameter at depth of first event
  d=depth1+cthk
  call s4rapars (d, del(ista),iph,alph2,beta2,rapar(ista),ptoa(ista))
  az=fnrad(azd(ista))
  rp=rapar(ista)                         'ray parameter for phase
  if kpwave then rpp=rp else rpp=rp/vps  'P ray parameter

'-- Get ACOR, geometrical spreading and reciever response
'--   First, at reference dist DEL0, get ref. correction GSP0
  call geomspr(del0,rpp,rho2,rhor,alph2,alphr,gspr) 
  call recvresp (kpwave,rpp,alphr,betar,acor)
    gsp0=acor*gspr

'------ Now, at actual delta
   delta=del(ista)
   call geomspr(delta,rpp,rho2,rhor,alph2,alphr,gspr) 

'--- GCORSTA is amplitude correction to data to move station to delta=DEL0
     gcorsta=gsp0/(acor*gspr)
     for i=1 to ninv : odat(i)=odat(i)*gcorsta : next
     gcor(ista)=gcorsta

'******************* SOURCE LOOP

  for isor = 1 to nsor
 
     lastsor= (isor=nsor)

     strike=sorpar(1,isor)
     dip =  sorpar(2,isor)
     rake = sorpar(3,isor)
     dephs =sorpar(4,isor) 
     delort=sorpar(5,isor)
     delpos=sorpar(6,isor)
     delphi=sorpar(7,isor)

'-- Calculate radiation pattern F(3) and its derivatives DF(3,3) w.r.t. mechanism
'  Indexing for F(i) and DF(i,j):  i=1,2,3 for P down, P up, SV up or SH down, 
'   SH up;  j=1,2,3 for deriv's w.r.t strike, dip, rake.
     
  call s4radpat (mkstat,kpwave,strike,dip,rake,az,rp,alph2,beta2,rho2,f(),df(),vslow()) 

'--- Adjust amplitudes and derivatives to moment and reference geom. spreading 
    xamp=tam(isor)*ucor*gsp0
    for i=1 to 3 
      f(i)=f(i)*xamp 
     if mkstat then for j=1 to 3 : df(i,j)=df(i,j)*xamp : next
    next

'--- Get delay times for pP and sP, or sS
'--  PDLY0(1) = delay for pP,sS,  PDLY0(2) = delay for sP
    call deltime (kpwave,rpp,alph2,beta2,dephs,pdly0())

'-- event delay and position delay relative to event 1
'       time offset         position                   depth difference
  evdly = delort - rp * delpos * cos(delphi-az) + (depth1-dephs)*vslow(1)

    evdly = evdly/dt                  'change to units of DT
    pdly(1)=evdly                     'direct phase
    pdly(2)=pdly0(1)/dt+evdly         'pP or sS
    pdly(3)=pdly0(2)/dt+evdly         'sP

    if pdly(1)<0 or pdly(2)<0 or pdly(3)<0 then
      call scprint (22,10,"*** Phase arrival time is negative ***")
      goto 2520
    end if

'-- loop for each phase:  calculate full seismograms and derivatives

  for ip=1 to nph+1
      kflag= (ip=1)  ' flag direct phase
      tau=tau(ip,ista)
      kinteg=(tau/dt=int(tau/dt)) 'if TAU is integer multiple of DT

' --- get elementary seismogram
   redim es0(2*nes) 
   ipm1=ip-1
   if not kflag then call s4esio (1, ista, ipm1, nes, es0() )

'------ generate synthetic: --------

'--- For each STF element set ITT, delay for start of elem. seis., and ATAU(i)
'---  interpolate for each delay time: source+phase+STF delays

970 for idt=1 to ntau(isor)

   gosub CHECKIN

   atau=atau(idt,isor)
   delay= pdly(ip)*dt + (idt-1)*tau    '--- delay in seconds for start of ES
   itt=int(delay/dt)                   '--- point before start of ES(tau,ph)
   itdly=itt-int(pdly(ip))             '--- start point after start of phase
   dd=delay/dt-itt : omdd=1-dd

990
'--- loop through the seismogram and interpolate values
        lim=nesp-itt-1
        if lim<1 then 1350
        f=f(ip)
        if not kflag then nn=nphes + ip-2

        if not kinteg or idt=1 then
          if kflag then
            for i=1 to lim : es2(i)=sp(i+isprec1)*dd + sp(i+1+isprec1)*omdd : next i
          else
            for i=1 to lim : es2(i)=es0(i)*dd + es0(i+1)*omdd : next i
          end if
        end if
995
'-- ES2 is interpolated so should be added to SS such that ES2(1) adds to
'--  SS( int(delay/dt) +1) or ITT+1

       if atau<>0 then 
          ii=itt
          iid=itdly
          for i=1 to lim
            ii=ii+1
            iid=iid+1
            ess=es2(i)*atau
            es3(iid,ip)=es3(iid,ip) + ess
            ss(ii) = ss(ii) + ess*f
          next i
       end if
997          
'--- calculate STF derivatives 
'    SS is linear in these, so just do SS without STF elem 
       if stfdamp(isor)>0 and mkstat and not noninv then
          ii=itt
          kp=ktau(idt,isor)
          for i=1 to lim
              ii=ii+1
              d(ii,kp)=d(ii,kp) + f*es2(i)
          next i
       end if

1350  next idt

    next ip

'-- SS, the synthetic seismogram, is now formed

'-- Setup and do an inversion
'-- set up matrix of derivatives D

 if (not mkstat) or (not ievflag(isor)) then 1920 

'-- Accumulate derivatives:  
'    IT = index for point before first point of phase
'    ES3(i,ip) = complete elementary seismogram for phase IP
'    KPAR(ip) = index in jacobian for parameter ip

'-- derivatives for strike, dip, rake
'     as stored in DF array, *ATAU*SP [or EP], first for direct, then refl.
    
    for ip=1 to nph+1
       gosub CHECKIN
       it=int(pdly(ip)) : lim=ninv-it
         if lim> 1 then 

          for kvar=1 to 3
            if damp(kvar,isor)>0 then

              kp=kpar(kvar,isor) 
              df=df(ip,kvar)
              for i=1 to lim
                k=it+i
                d(k,kp)=d(k,kp) +df*es3(i,ip)
              next i

            end if   '-- damp
          next kvar
        end if    '-- lim
    next ip

'--depth derivatives for pP and sP
' IT = start of phase, LIM = # pts to do in phase, NEQ = current pt in SS
' Calculated by finite differencing w/ time and multiplying by vertical slowness

  if damp(4,isor) > 0 then 
     kp=kpar(4,isor)

     for ip=1 to nph
        jp=ip+1             'the PDLY and F index for reflected phases
        it=int(pdly(jp)) : lim=ninv-it-1
          if lim<1 then 1610
            neq=it+1 : f=f(jp)*vslow(ip) : f1=f/(2*dt)
            d(neq,kp)=d(neq,kp) - f*(es3(2,jp)-es3(1,jp))/dt          
            if lim<2 then 1610
              for i=2 to lim
                 neq=it+i
                 d(neq,kp)=d(neq,kp) - (es3(i+1,jp)-es3(i-1,jp))*f1   'DEPTH
              next i
1610 next ip
  end if


'----- derivatives for origin time and position of secondary events
  
 for kvar= 5 to 7
  gosub CHECKIN
  if damp(kvar,isor) > 0 then 
     kp=kpar(kvar,isor)

     for ip=1 to nph+1
        it=int(pdly(ip)) : lim=ninv-it-1
          if lim<1 then 1620
'                 du/dDELORT = -du/dt
            if kvar=5 then f = -f(ip)
'                 du/dDELPOS = du/dt rp cos(DELPHI- Az)
            if kvar=6 then f =  f(ip)*rp*cos(delphi-az)
'                 du/dDELPHI = -du/dt rp DELPOS sin(DELPHI-Az)
            if kvar=7 then f = -f(ip)*rp*delpos*sin(delphi-az)

            neq=it+1 : f1=f/(2*dt)
            d(neq,kp)=d(neq,kp) + f*(es3(2,ip)-es3(1,ip))/dt
            if lim<2 then 1620
               for i=2 to lim
                 neq=neq+1
                 d(neq,kp)=d(neq,kp) + (es3(i+1,ip)-es3(i-1,ip))*f1
               next i
1620    next ip
  end if
 next kvar

'=====  Derivatives are now formed in matrix D ======

1920 next isor 
'=== end of loop for sources ==========


'====  Do Statistics :  ========

  if (not noninv) and mkstat then
'-- do statistics for this seismogram
     for i=1 to ninv : tmp(i)=ss(i)*xmag : next
     call makestat(tmp(),odat(),wt,ninv,invflag,r(),stavar(ista),synorm) 

'--- weight derivatives with station weights, and magnification
'     and normalize by synthetic rms, if just doing shape inversion 

        x= xmag*wt/synorm
        for  j=1 to npar    
           for i=1 to ninv : d(i,j)=d(i,j)*x : next
        next

'---  Add this station to matrix B=DtD, and vector Z=DtR
     call buildb (npar, ninv, d(),r(),z(),b())
  end if    '--- doing statistics

'--- calculate rms and write synthetics to file or array on last iteration
    if itlast then      
      ipof=ipf2(ista)-1
      if ninv(ista)=0 then n=nes(ista) else n=ninv(ista)
      if ipof+n>ndat(ista) then n=ndat(ista)-ipof
        call s4synio (1,0,ista,n,ipof,odat(),dext(ista),datrms(ista))
      s2=0 : for i=1 to n : s2=s2+ss(i)*ss(i) : next
       synrms(ista)= sqr(s2)*xmag
      call s4synio (0,1,ista,nes(ista),0,ss(),amax,dummy) 
    end if

2090 next ista   '------ End of station loop -------

'-- set up statistics  
if mkstat then 
     varnowt=res2/neq2   'unweighted variance
     totsum0=0 : vardat0=0 : varres0=0

'-- get variances of data and residuals for each phase-data type
     for i=1 to nphtype
      if sumwtph(i)>0 then 
         totsum0=totsum0+sumwtph(i)
         vardat0=vardat0+datvar(i)
         varres0=varres0+resvar(i)
         datvar(i)=datvar(i)/sumwtph(i)
         resvar(i)=resvar(i)/sumwtph(i)
      else
         datvar(i)=0.0
         resvar(i)=0.0
      end if
     next

'-- VARDAT and VARRES are the variances of the data and residuals
   if totsum0>0 then 
     vardat=vardat0/totsum0
     varres=varres0/totsum0
   end if
      dvar(itno)=varres

'-- DATERR is estimated unweighted standard error in residuals
    daterr=sqr(res2/(neq2-npar))

    nrow=neq2 : ncol=npar  '-- total # data, params

'--- Damping: set least squares dampers scaled to data variances on 
'      first iteration.  THETA = damping factor, DIAGWT has parm variance
    if (theta>0) or ipcflag then gosub ADDLSDAMP

'----- solve for adjustments:  Gauss-Jordan inversion for Binv and (Binv)Z
'      ISING flags a singular matrix
     call s4gjinv(npar,b(),z(),d,0,ising) 
      if ising then call scprint (22,10,"*** Singular Matrix *** ") : goto 2520

'---- check STF positivity, and damp out negative STF elements
   if ipcflag and (not itlast) then
     ik=kfalse
     for i=1 to nsor 
       if stfdamp(i)>0 then ik=ktrue 
     next
     if ik then gosub POSITIVT
   end if

2120
'----- calculate standard errors of parameters
'-- for orientation
    for isor=1 to nsor
     for i= 1 to 7
         erpar(i,isor)=0.0
         kp=kpar(i,isor)
         if damp(i,isor)>0 then erpar(i,isor)=daterr * sqr(b(kp,kp))
         if i<4 or i=7 then erpar(i,isor)=fndeg(erpar(i,isor)) 
     next

'-- for source time function
     ermom(isor)=0.0
     x=0 : x2=0
     for i=1 to ntau(isor)
       ertau(i,isor)=0.0
       kp=ktau(i,isor)
       if stfdamp(isor)>0 then 
         ertau(i,isor) = daterr * sqr(b(kp,kp))
          for j=1 to ntau(isor)
           b=b(kp,ktau(j,isor))
           at2 = atau(i,isor)+z(kp)*stfdamp(isor)           
           x=x+b 
           if at2>0 then x2=x2+i*j*b
          next j
       end if
     next
     x=fnamax(x,0.0)
'-- standard error in duration
     ert95(isor)=4.*tau0*daterr*sqr(abs(x2))
     ermom(isor)=sqr(x)*tam(isor)*daterr 
   next isor

'-----  write events, one to a screen
for isor=1 to nsor
   iscreen=isor+1
    itline= ((itno-1) mod 8) +1
  call scprint1 (2+itline,1,blank$)
  call scvint   (2+itline,1,2,itno)
  call scprint1 (3+itline,1,blank$)
  call scprint1 (3+itline,1,"Err")
    il=4
  for i=1 to 7
     if (i<4 or i=7) then 
       x=fndeg(sorpar(i,isor)) 
     elseif i=4 then
       x=sorpar(i,isor)+cthk
     elseif (i=5 or i=6) then
       x=sorpar(i,isor)
     end if
       x=fnfix(x,2)
   call scvreal( 2+itline, il, 6,1, x)
       x=fnfix(erpar(i,isor),2)
   call scvreal( 3+itline, il, 6,1, x)
   il=il+6
  next

   call scvexp( 2+itline, 47, 5,2, tam(isor)) :  il=il+10
   x1=varnowt*varcor : x2=varres*varcor
   call scvexp (2+itline, il, 5,3, x1) :  il=il+9
   call scvexp (2+itline, il, 5,3, x2) :  il=il+9
     if vardat=0 then x=0 else x=varres/vardat*100
   call scvreal (2+itline, il, 5,1, x) 
   call scvexp (3+itline, 47, 5,2, ermom(isor))

'-- time function elements
   ialine=12+itline : ieline=13+itline : il=4
   if ntau(isor)>18 then ialine=13 : ieline=16

   call scprint1 (ieline-1, 1, "   ")
   call scvint (ialine,1,2,itno)

  for i=1 to ntau(isor)
      ix=atau(i,isor)*100
     call scvint(ialine, il, 4, ix)
      ix=ertau(i,isor)*100
     call scvint(ieline, il, 4, ix)
      il=il+4
  next
     call scprint1 (ieline, 1, "Err")

next isor

call scprint2 (npage)

end if

'-- if not last iteration
     if itlast then 2520

'---- adjust model: scale to dampers -------------
for isor=1 to nsor
  for i=1 to 7
    if damp(i,isor)>0 then
      kp=kpar(i,isor)
      sorpar(i,isor)=sorpar(i,isor)+z(kp)*damp(i,isor)
    end if
  next

'--- reset depth if above halfspace, and adjust strike, dip, rake to keep 
'      in bounds
    sorpar(4,isor)=fnamax(0.1, sorpar(4,isor))
      strike=fndeg(sorpar(1,isor))
      dip=fndeg(sorpar(2,isor))
      rk=fndeg(sorpar(3,isor))

       if dip>=90 then
          dip=180-dip
          strike=strike + 180.
          rk = 360-rk
       elseif dip<0 then
          dip=abs(dip)
          strike=strike+180.
          rk=rk+180.
       end if

      if not fninlim(strike,0,360) then strike = (1800.+strike) mod 360
      if not fninlim(rk,0,360) then rk = (1800.+rk) mod 360 
      sorpar(1,isor)=fnrad(strike)
      sorpar(2,isor)=fnrad(dip)
      sorpar(3,isor)=fnrad(rk) 

     delphi=fndeg(sorpar(7,isor))
     if not fninlim(delphi,0,360) then delphi = (1800.+delphi) mod 360 
     sorpar(7,isor)=fnrad(delphi)

'--- STF adjustments 
    if stfdamp(isor)>0 then
     for i=1 to ntau(isor)
       kp=ktau(i,isor)
       atau(i,isor)=atau(i,isor)+z(kp)*stfdamp(isor)
     next
    end if

'---- normalize source time function and adjust moment
 call s4stf (invflag,tau0,isor,ntau(),atau(),tam(),tcent,t95) 

'-- adjust moment for shape inversion by the average of the RMS ratios
 if not invflag then
   if sumdswt>0.0 then avds=sumdos/sumdswt else avds=1.0
   tam(isor)=tam(isor)*avds
 end if

next isor

'-- check for input
    gosub CHECKIN
    totime = int(timer-time1)

'-- check on percent variance reduction
 if itno>1 and not itlast then
  varred = (dvar(itno-1) - dvar(itno))/dvar(itno-1) * 100

  if varred < varmin then 
       ivarstop=ktrue 
       call scprint (23,21,"Variance reduction below cutoff")
   else
       ivarstop=kfalse
       call scprint (23,21,"Variance reduction ") : print fnfix(varred,1);"%  ";
   end if

  end if

 next itno
'---- End iteration loop

2520 call cline (23,55)
     call scprintc(23,55," Press any key...   ")
     call inlet(kfalse, d$, klen, n, kascii)
     if fninlim(n,1,nsor) then n=n+1 : gosub NEWPAGE : goto 2520
     if d$="h" then n=1 : gosub NEWPAGE : goto 2520

    itf=itf+1
    if open2 then close 2

2525 chain SYNDIR$+"SYN4MEN1"

'++++++++++++++++++++ END PROGRAM ++++++++++++++++++++

'******** SUBROUTINES **************
'========================================================================
POSITIVT:
'--- Subroutine to impose positivity on STF by increasing damping
'      The dampers on STF elements that went below STFMIN are incresed
'      by DAMPFAC, and the inversion is redone.  This process is repeated
'      until the constraint is achieved
'
     kdflag=kfalse : dampmax=1e37

'--- TAUDAMP is total damping on STF elements, initialized to theta or 1
  for isor=1 to nsor: for i=1 to ntau(isor) : taudamp(i,isor)=1 : next i,isor

6600
'--- start by checking if any negative elements exist, and setting them to
'     zero. Also check # of increases in damping
   kneg=kfalse
    for isor=1 to nsor
       for i=1 to ntau(isor)
         kp=ktau(i,isor)
          if kp>0 then atau2(i,isor)=atau(i,isor)+z(kp)*stfdamp(isor) else atau2(i,isor)=atau(i,isor)
          if atau2(i,isor)<stfmin then kneg=ktrue : atau(i,isor)=0.
       next i, isor

'--- if STF has no negative elements, or too much damping, quit subroutine
     if not kneg then return

     if itry>ntrys then
       call scprint (22,10,"STF negative, max damping reached")
       return
     end if

'    call cline(22,10)
'    call scprint (22,10,"STF elements negative: ")

'-- get original matrices and add least square dampers to parameters

       for i=1 to ncol
         z(i)=z2(i)
         for j=1 to i
           b(i,j)=b2(i,j)
         next j,i

 for i=1 to npar : b(i,i)=b(i,i)+theta*diagwt : next

'--- check STF and add dampers where needed
'      TAUDAMP is increased for negative STF elements. Limit damping to prevent 
'      overflow errors

  for isor=1 to nsor
   if stfdamp(isor)>0 then
     for i=1 to ntau(isor)
      kp=ktau(i,isor)
       if (atau2(i,isor) < stfmin) then 
'        print "(";i;",";isor;")";
        if (dampmax/taudamp(i,isor)) =< dampfac then kdflag=ktrue _
           else taudamp(i,isor)=taudamp(i,isor)*dampfac 
       end if 
      if taudamp(i,isor)>1 then b(kp,kp)=b(kp,kp)+taudamp(i,isor)
     next i
  end if
 next isor

'--- re-invert matrix
   call s4gjinv(npar,b(),z(),d,0,ising) 

'--- quit if too much damping
  if kdflag then call scprint (22,10,"STF negative, max damping reached"): goto 6900

'--- restart subroutine to check if STF is now in acceptable bounds
     goto 6600
     
6900 return
'========================================================================
ADDLSDAMP: '---- Subroutine to add least squares damping
'  Scales damping to estimates of residual variance from first iteration
'  Input:  THETA, damping factor
'          B, the matrix being damped
'
'--- store the B (AtA) matrix and the Z (AtR) data array aside
     for i=1 to ncol
       z2(i)=z(i)
       for j=1 to i
         b2(i,j)=b(i,j)
       next
     next

'-- on first iteration, store residual variance
     if itno=1 then diagwt=varres

7100 '--- add damping to diagonal of B
     if theta>0 then  
         for i=1 to ncol : b(i,i)=b(i,i)+theta*diagwt : next
      end if

return
'========================================================================
PRTIME:
  locate 23,1: print "Time = ";int(timer-time1);"/";totime;"s"
  call scprint (23,56,"Station "+fnstanm$(ista)+" Phase "+fnkphlab$(kphtype(ista)))
 return
'========================================================================
NEWPAGE:
      if n<>npage then call scprint2(n) : npage=n
 return
'========================================================================
sub buildb (npar, ninv, d(2),r(1),z(1),b(2) ) static

'---- Subroutine to build B=DtD and Z=DtR.
'          Only lower-triangle of B stored.
 for k=1 to npar
  for l=1 to k
   for i=1 to ninv
     b(k,l)=b(k,l) + d(i,l)*d(i,k)
   next
  next
   for i=1 to ninv
    z(k)=z(k) + r(i)*d(i,k)
   next
 next
end sub
'========================================================================
sub deltime (kpwave,rpp,alph,beta,dephs,pdly0(1)) static
'---------- Subroutine to get upgoing P and S delay times in the halfspace
'               ALPH, BETA - source structure
'               DEPHS - source depth in halfspace
'        Output:  PDLY0(1) = delay for pP or sS
'                 PDLY0(2) = delay for sP 

    if kpwave then vel=alph else vel=beta
    z=fnasin(rpp*alph)  'P or SH angle
    y=fnasin(rpp*beta)  'SV angle

    putime=dephs/(vel*cos(z))        'upgoing P or SH time
    sutime=dephs/(beta*cos(y))       'upgoing SV time
    rx=dephs*tan(z) : svx=dephs*tan(y)    'horizontal distances traveled
   
    pdtime=(putime-2*sin(z)*rx/vel)      'P or SH down corrected to wavefront
    sdtime=(putime-(rx+svx)*sin(z)/vel)  'P down in sP phase

    pdly0(1)=(putime+pdtime)
    pdly0(2)=(sutime+sdtime)
end sub
'========================================================================
sub geomspr(delta,rpp,rho2,rhor,alph2,alphr,gspr) static

'--------- Subroutine for geometrical spreading and receiver crust response
'     Output: GSPR, just the geometrical spreading factor
'
     shared dgs(), cgs()

 if fninlim(delta,30,90) then
'--- get gspr from delta
     for i=2 to 16  'get the distance index
       if dgs(i)>delta then 
        delx=delta-dgs(i-1)
        dg=cgs(i)-cgs(i-1): dd=dgs(i)-dgs(i-1)
        gspr=cgs(i-1)+delx*dg/dd 
        exit for
       end if
     next
 elseif delta < 30. then
     delx=delta-30: dg=cgs(2)-cgs(1)
     gspr=cgs(1)+delx*dg/2.5
 elseif delta > 90. then
     delx=delta-90: dg=cgs(16)-cgs(15)
     gspr=cgs(16)+delx*dg/5.0
 end if

'--- adjust GSPR for appropriate source and receiver velocities
     rpt=rpp: rpt2=rpt^2
     a2=1/(6.5)^2: as2=1/alph2^2: ar2=1/alphr^2
     a=(rho2/rhor)*alph2^2/alphr^2
     c=a*(a2-rpt2)/(sqr(as2-rpt2)*sqr(ar2-rpt2))
     gspr=sqr(c)*gspr
end sub
'========================================================================
sub recvresp (kpwave, rpp, alphr, betar, acor) static
'---Receiver half-space response:  include free-surface effect at receiver
' RPP= ray parameter       ALPHR, BETAR = Vp, Vs of receiver structure
' ACOR = amplitude correction

     acor=2: if not kpwave then exit sub
     aipr=fnasin(rpp*alphr)
     aisr=fnasin(rpp*betar)
     dd=cos(2*aisr)^2+betar^2/alphr^2*sin(2*aipr)*sin(2*aisr)
     acor=2*cos(aipr)*cos(2*aisr)/dd
end sub
'========================================================================
sub makestat(ss(1),odat(1),wt,ninv,invflag,r(1),stavar,synorm) static
'---- routine to calculate residuals and update statistics for an individual 
'     seismogram

  shared wres2, res2, sumwt, ista, gcor()
  shared datvar(), resvar(), sumwtph(), kphtype()
  shared sumdos, sumdswt

'-- sums of amplitudes squared
      sums=0 : sumd=0
      for i=1 to ninv 
        sums=sums + ss(i)*ss(i)
        sumd=sumd + odat(i)*odat(i)
      next

'-- if doing shape inversion calculate normalizations SYNORM and DATNORM 
    synorm=1.0 : datnorm=1.0
    if not invflag then 
      synorm=sqr(sums) : datnorm=sqr(sumd)
      if synorm<>0 then sumdos=sumdos+wt*datnorm/synorm : sumdswt=sumdswt+wt
    end if

'-- Form residuals with normalizations, weighted residuals WRES,
'    and various sums of squares and variances

     wsrms=0 : srms=0 : sumd=0
      for i=1 to ninv
         odat=odat(i)/datnorm : sumd=sumd+odat*odat*wt*wt
         res=odat-ss(i)/synorm
         wres=res*wt : r(i) = wres
         res2=res2+res*res
         wres2=wres2+wres*wres
         srms=srms+res*res         'unweighted station res
         wsrms=wsrms+wres*wres     'weighted station res
      next

      sumwt=sumwt+ninv*wt  

'-- weighted data and residual variancess by PDT
      sumwtph(kphtype(ista))=sumwtph(kphtype(ista))+ninv*wt 
      resvar(kphtype(ista))=resvar(kphtype(ista))+wsrms
      datvar(kphtype(ista))=datvar(kphtype(ista))+sumd
      stavar=srms/ninv
end sub

'========================================================================
CHECKIN:
'-- check for input of new source to be displayed on screen
   d$=lcase$(inkey$) : n=val(d$)
   if fniquit(d$) then 2520
   if fninlim(n,1,nsor) then n=n+1 : gosub NEWPAGE
   if d$="h" then n=1 : gosub NEWPAGE
   gosub PRTIME
return

'========================================================================
'-- screen subroutines

sub scprint1(ix,iy,text$) static
 shared iscreen, liner$(), ncharm
  ichar = (ix-1)*80 + iy
  if ichar<= ncharm then mid$(liner$(iscreen),ichar)=text$
end sub
'-----------------------------------------------------------------------
sub scprint2 (k) static
 shared iscreen, liner$(), ncharm
 for i=1 to 21
  locate i,1 : print mid$(liner$(k),80*(i-1)+1,80);
 next
  if k>1 then locate 1,7 : print k-1;
end sub
'-----------------------------------------------------------------------
sub scvint( ix, iy, klen, kvar) static
 shared iscreen, liner$(), ncharm
  ichar = (ix-1)*80 + iy
  text$ = string$(klen,32)+str$(kvar)
  text$=right$(text$,klen)
  if ichar<= ncharm then mid$(liner$(iscreen),ichar)=text$
end sub
'-----------------------------------------------------------------------
sub scvreal( ix, iy, klen, kfrac, varin) static
 shared iscreen, liner$(), ncharm
  ichar = (ix-1)*80 + iy
  var=varin

  v$=str$(var) : kE=instr(v$,"E")
    if kE>0 then
     iex=val(mid$(v$,kE+1,3))
     if -iex>kfrac then var=0 else var=val(mid$(v$,1,kE-1)) * 10^iex
    end if

  text$ = "              "+str$(var) : kdot=instr(text$,".")
  if kdot=0 then text$=text$+"."     : kdot=instr(text$,".")
  text$=text$+"000000000"
  k1=kdot-(klen-kfrac-1)
  text$=mid$(text$,k1,klen)

  if ichar<=ncharm then mid$(liner$(iscreen),ichar)=text$
end sub
'-----------------------------------------------------------------------
sub scvexp ( ix, iy, klen, kfrac, var) static
 shared iscreen, liner$(), ncharm

3700  kend=klen+iy-1
  ichar = (ix-1)*80 + iy

  if var<>0. then
3701 v=log(abs(var))/log(10)
3711 iexp=int(v)
  else
   iexp = 0
  end if

  v=var/(10^iexp)
3702 exp$=str$(iexp)
3703 if iexp>=0 then exp$=right$(exp$,len(exp$)-1)
  exp$="E"+exp$

  text$ = "            "+str$(v)
3704 kdot=instr(text$,".")
3705 if kdot=0 then text$=text$+"." : kdot=instr(text$,".")
  text$=text$+"000000000"
  k1=kdot-(klen-kfrac-1)
3706 text$=mid$(text$,k1,klen)+exp$

3707 if ichar<= ncharm then mid$(liner$(iscreen),ichar)=text$
end sub
'========================================================================
rem $INCLUDE: 's4rapars'
rem $INCLUDE: 's4stf'
'rem $INCLUDE: 's4synio'
rem $INCLUDE: 's4gjinv'
rem $INCLUDE: 's4radpat'
'========================================================================
